@@ -46,7 +46,7 @@
 char sandbox_lib[SB_PATH_MAX];
 
 typedef struct {
-	int show_access_violation;
+	bool show_access_violation;
 	char **prefixes[5];
 	int num_prefixes[5];
 #define             deny_prefixes     prefixes[0]
@@ -63,14 +63,11 @@ typedef struct {
 } sbcontext_t;
 
 static sbcontext_t sbcontext;
-static char **cached_env_vars;
-volatile int sandbox_on = 1;
-static int sb_init = 0;
-static int sb_path_size_warning = 0;
+static char *cached_env_vars[MAX_DYN_PREFIXES];
+volatile bool sandbox_on = true;
+static bool sb_init = false;
 
 static char *resolve_path(const char *, int);
-static int write_logfile(const char *, const char *, const char *,
-						 const char *, const char *, bool);
 static int check_prefixes(char **, int, const char *);
 static void clean_env_entries(char ***, int *);
 static void init_context(sbcontext_t *);
@@ -86,17 +83,13 @@ void libsb_fini(void)
 {
 	int x;
 
-	sb_init = 0;
+	sb_init = false;
 
-	if (NULL != cached_env_vars) {
-		for (x = 0; x < MAX_DYN_PREFIXES; ++x) {
-			if (NULL != cached_env_vars[x]) {
-				free(cached_env_vars[x]);
-				cached_env_vars[x] = NULL;
-			}
+	for (x = 0; x < MAX_DYN_PREFIXES; ++x) {
+		if (NULL != cached_env_vars[x]) {
+			free(cached_env_vars[x]);
+			cached_env_vars[x] = NULL;
 		}
-		free(cached_env_vars);
-		cached_env_vars = NULL;
 	}
 
 	for (x = 0; x < MAX_DYN_PREFIXES; ++x)
@@ -113,12 +106,12 @@ void libsb_init(void)
 	mtrace();
 #endif
 
-	sb_set_open(libsb_open);
+	sb_set_open(sb_unwrapped_open);
 
 	/* Get the path and name to this library */
 	get_sandbox_lib(sandbox_lib);
 
-//	sb_init = 1;
+//	sb_init = true;
 
 	errno = old_errno;
 }
@@ -245,16 +238,15 @@ char *egetcwd(char *buf, size_t size)
 {
 	struct stat st;
 	char *tmpbuf, *oldbuf = buf;
-	int old_errno;
 
 	/* Need to disable sandbox, as on non-linux libc's, opendir() is
 	 * used by some getcwd() implementations and resolves to the sandbox
 	 * opendir() wrapper, causing infinit recursion and finially crashes.
 	 */
-	sandbox_on = 0;
+	sandbox_on = false;
 	errno = 0;
-	tmpbuf = libsb_getcwd(buf, size);
-	sandbox_on = 1;
+	tmpbuf = sb_unwrapped_getcwd(buf, size);
+	sandbox_on = true;
 
 	/* We basically try to figure out if we can trust what getcwd()
 	 * returned.  If one of the following happens kernel/libc side,
@@ -264,14 +256,12 @@ char *egetcwd(char *buf, size_t size)
 	 *  - Whatever I forgot about
 	 */
 	if ((tmpbuf) && (errno == 0)) {
-		old_errno = errno;
-		lstat(buf, &st);
-
-		if (errno == ENOENT) {
-		  	/* If getcwd() allocated the buffer, free it. */
-			if (NULL == oldbuf)
-				free(tmpbuf);
+		save_errno();
+		if (!lstat(buf, &st))
+			/* errno is set only on failure */
+			errno = 0;
 
+		if (errno == ENOENT)
 			/* If lstat() failed with eerror = ENOENT, then its
 			 * possible that we are running on an older kernel
 			 * which had issues with returning invalid paths if
@@ -280,8 +270,8 @@ char *egetcwd(char *buf, size_t size)
 			 * what the issue is.
 			 */
 		  	errno = ENAMETOOLONG;
-			return NULL;
-		} else if (errno != 0) {
+
+		if (errno) {
 		  	/* If getcwd() allocated the buffer, free it. */
 			if (NULL == oldbuf)
 				free(tmpbuf);
@@ -294,7 +284,7 @@ char *egetcwd(char *buf, size_t size)
 			return NULL;
 		}
 
-		errno = old_errno;
+		restore_errno();
 	} else if (errno != 0) {
 
 		/* Make sure we do not return garbage if the current libc or
@@ -307,8 +297,8 @@ char *egetcwd(char *buf, size_t size)
 }
 
 #define _SB_WRITE_STR(str) SB_WRITE(logfd, str, strlen(str), error)
-static int write_logfile(const char *logfile, const char *func, const char *path,
-						 const char *apath, const char *rpath, bool access)
+static bool write_logfile(const char *logfile, const char *func, const char *path,
+                          const char *apath, const char *rpath, bool access)
 {
 	struct stat log_stat;
 	int stat_ret;
@@ -329,7 +319,7 @@ static int write_logfile(const char *logfile, const char *func, const char *path
 		S_IRUSR | S_IWUSR | S_IRGRP | S_IROTH);
 	if (logfd == -1) {
 		SB_EERROR("ISE:write_logfile ", "unable to append logfile\n");
-		return -1;
+		goto error;
 	}
 
 	if (0 != stat_ret)
@@ -384,37 +374,38 @@ static int write_logfile(const char *logfile, const char *func, const char *path
 		_SB_WRITE_STR("<unable to read " PROC_SELF_CMDLINE ">");
 	_SB_WRITE_STR("\n");
 
-	return 0;
+	return true;
 
  error:
-	return -1;
+	return false;
 }
 
 static void init_context(sbcontext_t *context)
 {
 	memset(context, 0x00, sizeof(*context));
-	context->show_access_violation = 1;
+	context->show_access_violation = true;
 }
 
 static void clean_env_entries(char ***prefixes_array, int *prefixes_num)
 {
-	int old_errno = errno;
-	int i = 0;
+	if (*prefixes_array == NULL)
+		return;
 
-	if (NULL != *prefixes_array) {
-		for (i = 0; i < *prefixes_num; i++) {
-			if (NULL != (*prefixes_array)[i]) {
-				free((*prefixes_array)[i]);
-				(*prefixes_array)[i] = NULL;
-			}
+	size_t i;
+	save_errno();
+
+	for (i = 0; i < *prefixes_num; ++i) {
+		if (NULL != (*prefixes_array)[i]) {
+			free((*prefixes_array)[i]);
+			(*prefixes_array)[i] = NULL;
 		}
-		if (NULL != *prefixes_array)
-			free(*prefixes_array);
-		*prefixes_array = NULL;
-		*prefixes_num = 0;
 	}
+	if (NULL != *prefixes_array)
+		free(*prefixes_array);
+	*prefixes_array = NULL;
+	*prefixes_num = 0;
 
-	errno = old_errno;
+	restore_errno();
 }
 
 #define pfx_num		(*prefixes_num)
@@ -435,7 +426,7 @@ static void init_env_entries(char ***prefixes_array, int *prefixes_num, const ch
 	if (NULL == prefixes_env) {
 		/* Do not warn if this is in init stage, as we might get
 		 * issues due to LD_PRELOAD already set (bug #91431). */
-		if (1 == sb_init)
+		if (sb_init)
 			fprintf(stderr,
 				"libsandbox:  The '%s' env variable is not defined!\n",
 				env);
@@ -557,7 +548,7 @@ static int check_access(sbcontext_t *sbcontext, int sb_nr, const char *func, con
 		/* If we are here, and still no joy, and its the access() call,
 		 * do not log it, but just return -1 */
 		if (sb_nr == SB_NR_ACCESS_RD) {
-			sbcontext->show_access_violation = 0;
+			sbcontext->show_access_violation = false;
 			goto out;
 		}
 	}
@@ -568,7 +559,10 @@ static int check_access(sbcontext_t *sbcontext, int sb_nr, const char *func, con
 	    sb_nr == SB_NR_CREAT64     ||
 	    sb_nr == SB_NR_MKDIR       ||
 	    sb_nr == SB_NR_MKNOD       ||
+	    sb_nr == SB_NR_MKNODAT     ||
+	    sb_nr == SB_NR__XMKNOD     ||
 	    sb_nr == SB_NR___XMKNOD    ||
+	    sb_nr == SB_NR___XMKNODAT  ||
 	    sb_nr == SB_NR_MKFIFO      ||
 	    sb_nr == SB_NR_LINK        ||
 	    sb_nr == SB_NR_SYMLINK     ||
@@ -671,7 +665,7 @@ static int check_access(sbcontext_t *sbcontext, int sb_nr, const char *func, con
 		 * behavior category, so no need to check the syscall.
 		 */
 		char *dname_buf = strdup(resolv_path);
-		int aret = access(dirname(dname_buf), F_OK);
+		int aret = sb_unwrapped_access(dirname(dname_buf), F_OK);
 		free(dname_buf);
 		if (aret) {
 			result = 1;
@@ -683,7 +677,7 @@ static int check_access(sbcontext_t *sbcontext, int sb_nr, const char *func, con
 		if (1 == retval) {
 			/* Is a known access violation, so deny access,
 			 * and do not log it */
-			sbcontext->show_access_violation = 0;
+			sbcontext->show_access_violation = false;
 			goto out;
 		}
 
@@ -696,7 +690,7 @@ static int check_access(sbcontext_t *sbcontext, int sb_nr, const char *func, con
 		if (len > 4) {
 			const char *py = resolv_path + len - 4;
 			if (!strcmp(py, ".pyc") || !strcmp(py, ".pyo")) {
-				sbcontext->show_access_violation = 0;
+				sbcontext->show_access_violation = false;
 				goto out;
 			}
 		}
@@ -704,7 +698,7 @@ static int check_access(sbcontext_t *sbcontext, int sb_nr, const char *func, con
 		/* If we are here, and still no joy, and its the access() call,
 		 * do not log it, but just return -1 */
 		if (sb_nr == SB_NR_ACCESS_WR) {
-			sbcontext->show_access_violation = 0;
+			sbcontext->show_access_violation = false;
 			goto out;
 		}
 	}
@@ -715,14 +709,19 @@ out:
 	return result;
 }
 
+/* Return values:
+ *  0: failure, caller should abort
+ *  1: things worked out fine
+ *  2: things worked out fine, but the errno should not be restored
+ */
 static int check_syscall(sbcontext_t *sbcontext, int sb_nr, const char *func, const char *file)
 {
 	char *absolute_path = NULL;
 	char *resolved_path = NULL;
-	char *log_path = NULL, *debug_log_path = NULL;
+	char *log_path, *debug_log_path;
 	int old_errno = errno;
-	int result = 1;
-	int access = 0, debug = 0, verbose = 1;
+	int result;
+	bool access, debug, verbose;
 
 	absolute_path = resolve_path(file, 0);
 	if (NULL == absolute_path)
@@ -731,48 +730,39 @@ static int check_syscall(sbcontext_t *sbcontext, int sb_nr, const char *func, co
 	if (NULL == resolved_path)
 		goto error;
 
+	verbose = is_env_on(ENV_SANDBOX_VERBOSE);
 	log_path = getenv(ENV_SANDBOX_LOG);
-	if (is_env_on(ENV_SANDBOX_DEBUG)) {
+	debug = is_env_on(ENV_SANDBOX_DEBUG);
+	if (debug)
 		debug_log_path = getenv(ENV_SANDBOX_DEBUG_LOG);
-		debug = 1;
-	}
-
-	if (is_env_off(ENV_SANDBOX_VERBOSE)) {
-		verbose = 0;
-	}
 
 	result = check_access(sbcontext, sb_nr, func, absolute_path, resolved_path);
 
-	if (1 == verbose) {
-		if ((0 == result) && (1 == sbcontext->show_access_violation)) {
-			SB_EERROR("ACCESS DENIED", "  %s:%*s%s\n",
-				func, (int)(10 - strlen(func)), "", absolute_path);
-		} else if ((1 == debug) && (1 == sbcontext->show_access_violation)) {
-			SB_EINFO("ACCESS ALLOWED", "  %s:%*s%s\n",
-				func, (int)(10 - strlen(func)), "", absolute_path);
-		} else if ((1 == debug) && (0 == sbcontext->show_access_violation)) {
-			SB_EWARN("ACCESS PREDICTED", "  %s:%*s%s\n",
-				func, (int)(10 - strlen(func)), "", absolute_path);
-		}
+	if (verbose) {
+		int sym_len = SB_MAX_STRING_LEN + 1 - strlen(func);
+		if (!result && sbcontext->show_access_violation)
+			SB_EERROR("ACCESS DENIED",   "  %s:%*s%s\n", func, sym_len, "", absolute_path);
+		else if (debug && sbcontext->show_access_violation)
+			SB_EINFO("ACCESS ALLOWED",   "  %s:%*s%s\n", func, sym_len, "", absolute_path);
+		else if (debug && !sbcontext->show_access_violation)
+			SB_EWARN("ACCESS PREDICTED", "  %s:%*s%s\n", func, sym_len, "", absolute_path);
 	}
 
-	if ((0 == result) && (1 == sbcontext->show_access_violation))
-		access = 1;
+	if ((0 == result) && sbcontext->show_access_violation)
+		access = false;
+	else
+		access = true;
 
-	if ((NULL != log_path) && (1 == access)) {
-		if (-1 == write_logfile(log_path, func, file, absolute_path,
-								resolved_path, (access == 1) ? 0 : 1)) {
-			if (0 != errno)
-				goto error;
-		}
+	if (log_path && !access) {
+		bool worked = write_logfile(log_path, func, file, absolute_path, resolved_path, access);
+		if (!worked && errno)
+			goto error;
 	}
 
-	if ((NULL != debug_log_path) && (1 == debug)) {
-		if (-1 == write_logfile(debug_log_path, func, file, absolute_path,
-								resolved_path, (access == 1) ? 0 : 1)) {
-			if (0 != errno)
-				goto error;
-		}
+	if (debug_log_path && debug) {
+		bool worked = write_logfile(debug_log_path, func, file, absolute_path, resolved_path, access);
+		if (!worked && errno)
+			goto error;
 	}
 
 	if (NULL != absolute_path)
@@ -791,25 +781,19 @@ error:
 		free(resolved_path);
 
 	/* The path is too long to be canonicalized, so just warn and let the
-	 * function handle it (see bug #94630 and #21766 for more info) */
-	if (ENAMETOOLONG == errno) {
-		if (0 == sb_path_size_warning) {
-			SB_EWARN("PATH LENGTH", "  %s:%*s%s\n",
-			      func, (int)(10 - strlen(func)), "", file);
-			sb_path_size_warning = 1;
-		}
-
-		return 1;
-	}
+	 * function handle it (see bugs #21766 #94630 #101728 #227947)
+	 */
+	if (ENAMETOOLONG == errno)
+		return 2;
 
 	/* If we get here, something bad happened */
 	SB_EERROR("ISE ", "Unrecoverable error: %s\n", strerror(errno));
 	abort();
 }
 
-int is_sandbox_on(void)
+bool is_sandbox_on(void)
 {
-	int result;
+	bool result;
 	save_errno();
 
 	/* $SANDBOX_ACTIVE is an env variable that should ONLY
@@ -819,28 +803,31 @@ int is_sandbox_on(void)
 	 * but not even in the sandbox shell.
 	 */
 	char *sb_env_active = getenv(ENV_SANDBOX_ACTIVE);
-	if ((1 == sandbox_on) &&
+	if (sandbox_on &&
 	    sb_env_active &&
 	    is_env_on(ENV_SANDBOX_ON) &&
 	    (0 == strcmp(sb_env_active, SANDBOX_ACTIVE)))
-		result = 1;
+		result = true;
 	else
-		result = 0;
+		result = false;
 	restore_errno();
 	return result;
 }
 
-int before_syscall(int dirfd, int sb_nr, const char *func, const char *file)
+/* Need to protect the global sbcontext structure */
+static pthread_mutex_t sb_syscall_lock = PTHREAD_MUTEX_INITIALIZER;
+
+bool before_syscall(int dirfd, int sb_nr, const char *func, const char *file)
 {
 	int old_errno = errno;
-	int result = 1;
+	int result;
 //	static sbcontext_t sbcontext;
 	char *at_file_buf = NULL;
 
 	if (file == NULL || file[0] == '\0') {
 		/* The file/directory does not exist */
 		errno = ENOENT;
-		return 0;
+		return false;
 	}
 
 	/* The *at style functions have the following semantics:
@@ -856,10 +843,11 @@ int before_syscall(int dirfd, int sb_nr, const char *func, const char *file)
 		file = at_file_buf;
 	}
 
-	if (0 == sb_init) {
+	pthread_mutex_lock(&sb_syscall_lock);
+
+	if (!sb_init) {
 		init_context(&sbcontext);
-		cached_env_vars = xcalloc(4, sizeof(char *));
-		sb_init = 1;
+		sb_init = true;
 	}
 
 	char *sb_env_names[4] = {
@@ -893,15 +881,15 @@ int before_syscall(int dirfd, int sb_nr, const char *func, const char *file)
 	}
 
 	/* Might have been reset in check_access() */
-	sbcontext.show_access_violation = 1;
+	sbcontext.show_access_violation = true;
 
 	result = check_syscall(&sbcontext, sb_nr, func, file);
 
+	pthread_mutex_unlock(&sb_syscall_lock);
+
 	if (at_file_buf)
 		free(at_file_buf);
 
-	errno = old_errno;
-
 	if (0 == result) {
 		if ((NULL != getenv(ENV_SANDBOX_PID)) && (is_env_on(ENV_SANDBOX_ABORT)))
 			kill(atoi(getenv(ENV_SANDBOX_PID)), SIGUSR1);
@@ -910,12 +898,13 @@ int before_syscall(int dirfd, int sb_nr, const char *func, const char *file)
 		 *        error to be returned (EINVAL for invalid mode for
 		 *        fopen() and co, ETOOLONG, etc). */
 		errno = EACCES;
-	}
+	} else if (result == 1)
+		errno = old_errno;
 
-	return result;
+	return result ? true : false;
 }
 
-int before_syscall_access(int dirfd, int sb_nr, const char *func, const char *file, int flags)
+bool before_syscall_access(int dirfd, int sb_nr, const char *func, const char *file, int flags)
 {
 	const char *ext_func;
 	if (flags & W_OK)
@@ -925,7 +914,7 @@ int before_syscall_access(int dirfd, int sb_nr, const char *func, const char *fi
 	return before_syscall(dirfd, sb_nr, ext_func, file);
 }
 
-int before_syscall_open_int(int dirfd, int sb_nr, const char *func, const char *file, int flags)
+bool before_syscall_open_int(int dirfd, int sb_nr, const char *func, const char *file, int flags)
 {
 	const char *ext_func;
 	if ((flags & O_WRONLY) || (flags & O_RDWR))
@@ -935,10 +924,10 @@ int before_syscall_open_int(int dirfd, int sb_nr, const char *func, const char *
 	return before_syscall(dirfd, sb_nr, ext_func, file);
 }
 
-int before_syscall_open_char(int dirfd, int sb_nr, const char *func, const char *file, const char *mode)
+bool before_syscall_open_char(int dirfd, int sb_nr, const char *func, const char *file, const char *mode)
 {
 	if (NULL == mode)
-		return 0;
+		return false;
 
 	const char *ext_func;
 	if ((*mode == 'r') && ((0 == (strcmp(mode, "r"))) ||
