@@ -32,8 +32,6 @@
 
 #endif
 
-#define GLIBC_MINOR __GLIBC_MINOR__
-
 #ifdef PATH_MAX
 # define SB_PATH_MAX PATH_MAX * 2
 # if (SB_PATH_MAX >= INT_MAX) || (SB_PATH_MAX < PATH_MAX)
@@ -51,10 +49,20 @@
 # define MAP_ANONYMOUS MAP_ANON
 #endif
 
-#ifndef HAVE_DECL_ENVIRON
+#if !HAVE_DECL_ENVIRON
 extern char **environ;
 #endif
 
+#ifndef HAVE_SIGHANDLER_T
+# if defined(HAVE_SIG_T)
+typedef sig_t sighandler_t;
+# elif defined(HAVE___SIGHANDLER_T)
+typedef __sighandler_t sighandler_t;
+# else
+#  error "your system blows chunks: no signal() type"
+# endif
+#endif
+
 /* If the system is old and does not support *at funcs, then define
  * it ourself.  Shouldn't matter based on how we use it.
  */
@@ -80,5 +88,6 @@ extern char **environ;
 # define weak_alias(_name, _aliasname) \
 	extern __typeof (_name) _aliasname __attribute__ ((weak, alias (#_name)));
 
+#define attribute_hidden __attribute__((visibility("hidden")))
 
 #endif
